# Board Bring-Up

Board Bring-Up is the firmware used for validating new hardware that has come directly from the contract manufacturing partner (CM).

> **Note**  
> This project is meant to be used with a locally installed version of the nRF Connect SDK (NCS).
>
> NCS v2.3.0

## Status

**Work In Progress**

## Installing / Getting started

### Dependencies

You need the toolchain for the nRF9160 installed first, available through **[nRF Connect for Desktop](https://www.nordicsemi.com/Products/Development-tools/nrf-connect-for-desktop)**, minimum `v4.0.0`.

Use the **Toolchain Manager** in **nRF Connect for Desktop** to install `nRF Connect SDK v2.3.0`.

One of the important tools installed with the SDK is Zephyr's **`west`** - a metatool for managing Git repositories and dependencies.

### Cloning the Git

You do not need to use `west init` to clone this repository.

Move to your preferred project directory and use `git clone` per usual, for example using SSH:

```bash
git clone git@ssh.dev.azure.com:v3/cm-one/nordic-sensor-platform/board-bring-up
```

### Directory Structure

The files and directories are structured according to the default structure of the Zephyr sample projects, and also incorporates guidelines from [the Pitchfork Layout](https://api.csswg.org/bikeshed/?force=1&url=https://raw.githubusercontent.com/vector-of-bool/pitchfork/develop/data/spec.bs#intro)

> **Git is set to ignore the `build` directories.**

```
. # Project Root
├── .vscode/    # could be ignored
├── boards/arm/CM11C0BC010007001/
|   ├── board.cmake
|   ├── CM11C0BC010007001_defconfig
|   ├── CM11C0BC010007001-pinctrl.dtsi
|   ├── CM11C0BC010007001.dts
|   ├── CM11C0BC010007001.yaml
|   ├── Kconfig.board
|   └── Kconfig.defconfig
├── build*/     # ignored by git
├── docs/
├── src/
|   └── main.c
├── tests/      # empty for now
├── cm1_module/
│   ├── drivers/
|   |   ├── gpio_ext/
|   |   |   ├── CMakeLists.txt
|   |   |   ├── gpio_tca9534.c
|   |   |   ├── Kconfig
|   |   |   └── Kconfig.tca9534
|   |   ├── CMakeLists.txt
|   │   └── Kconfig
│   ├── dts/bindings/
|   |   ├── gpio_ext/
|   |   |   └── ti,tca9534.yml
|   |   └── vendor-prefixes.txt
│   ├── zephyr/
|   │   └── module.yml
│   ├── CMakeLists.txt
│   └── Kconfig
├── CMakeLists.txt
├── prj.conf
└── README.md
```
_Git files like `.gitignore` are excluded from the explanation._

#### `boards/`

The `boards` directory contains the definitions of all the boards for which this project should be built. Each board gets its own directory inside of parent `boards`.

Each board must have the listed files defined and in its directory:
- `board.cmake`: boilerplate used by the toolchain to configure the SDK and the JLink
- `CM11C0BC010007001_defconfig`: board configurations that become part of the build, such as which SoC is used and other chip/board specific parameters.
- `CM11C0BC010007001-pinctrl.dtsi`: the devicetree source include file which defines the pin mapping on the Nordic SoC for controlling the communication busses. This is included by the board's devicetree source file.
- `CM11C0BC010007001.dts`: the devicetree source file for the board defines the pin mappings and devices that connect either externally or internally to the SoC.
- `CM11C0BC010007001.yaml`: boilerplate YAML file defining properties of the board.
- `Kconfig.board` and `Kconfig.defconfig`: boilerplate Kconfig files for the board used by the toolchain when this board is selected for a build.

The board to use for a build can be set in VS Code by defining a _Build Configuration_ in the **nRF Connect** extension for the project, or on the command line using the `west` command. 

#### `build*/`

Different build directories can be specified for each kind of board or configuration.

The files found in each build directory include not only the compiled object files and binaries, but also other important configuration files which must be generated first before building the application. These include combined Kconfig files and DTS files.

#### `docs/`

Designated to contain project documentation.

#### `src/`

Contains the actual application source and header files.

#### `tests/`

Reserved for source files related to tests for the project.

#### `cm1_module/`

We can define custom device drivers to be included in the devicetree and available to the application in several ways. This project uses the **out-of-tree** method.

Zephyr allows for the possibility to define extra modules, in which we can add our custom device drivers. The Zephyr toolchain expects a certain structure for a module and also certain CMake and Kconfig files.

Often times a device driver for a physical, external peripheral device will need a devicetree binding - this is a file used to tell the verification program what the expected parameters are for the device driver when the device is added to the board's devicetree.

The files and directories in this directory where cobbled togther using two sources of inspiration:
- [Out Of Tree Driver (NCS Example)](https://developer.nordicsemi.com/nRF_Connect_SDK/doc/2.3.0/zephyr/samples/application_development/out_of_tree_driver/README.html)
- [How to Build Drivers for Zephyr by Jared Wolff](https://interrupt.memfault.com/blog/building-drivers-on-zephyr)

## Developing

### Deploying / Flashing

Open the project with VS Code using the **nRF Connect** extensions that were downloaded with the toolchain.

Follow the instructions provided by Nordic for flashing the device.
