/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <stdio.h>
#include <string.h>

#include <zephyr/init.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
// #include <zephyr/drivers/gpio/gpio_sx1509b.h>
#include <zephyr/drivers/gpio.h>

#include <sys/types.h>
#include <zephyr/logging/log.h>
#include <zephyr/logging/log_ctrl.h>
#include <zephyr/sys/reboot.h>
#include <zephyr/dfu/mcuboot.h>

#include <zephyr/shell/shell.h>
#include <zephyr/shell/shell_rtt.h>

#include "mosh_defines.h"
#include "mosh_print.h"

LOG_MODULE_REGISTER(main_module, 4);

/***** Work queue and work item definitions *****/

#define MOSH_COMMON_WORKQ_PRIORITY CONFIG_MOSH_COMMON_WORKQUEUE_PRIORITY
K_THREAD_STACK_DEFINE(mosh_common_workq_stack, CONFIG_MOSH_COMMON_WORKQUEUE_STACK_SIZE);
struct k_work_q mosh_common_work_q;

/* Global variables */
const struct shell *mosh_shell;
struct k_poll_signal mosh_signal;

K_SEM_DEFINE(nrf_carrier_lib_initialized, 0, 1);

static void mosh_print_version_info(void)
{
#if defined(APP_VERSION)
	printk("\nMOSH version:       %s", STRINGIFY(APP_VERSION));
#else
	printk("\nMOSH version:       unknown");
#endif

#if defined(BUILD_ID)
	printk("\nMOSH build id:      v%s", STRINGIFY(BUILD_ID));
#else
	printk("\nMOSH build id:      custom");
#endif

#if defined(BUILD_VARIANT)
#if defined(BRANCH_NAME)
	printk("\nMOSH build variant: %s/%s\n\n", STRINGIFY(BRANCH_NAME), STRINGIFY(BUILD_VARIANT));
#else
	printk("\nMOSH build variant: %s\n\n", STRINGIFY(BUILD_VARIANT));
#endif
#else
	printk("\nMOSH build variant: dev\n\n");
#endif
}

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   1000

/* The devicetree node identifier for the "en_3v2" alias. */
#define EN_3V2_NODE DT_ALIAS(en_3v2)

#define USERLED_1 (0)

//static const struct gpio_dt_spec en_3v2 = GPIO_DT_SPEC_GET(EN_3V2_NODE, gpios);

//static const struct device *io_expander;

void main(void)
{
	//int ret;

	mosh_shell = shell_backend_rtt_get_ptr();

	struct k_work_queue_config cfg = {
		.name = "mosh_common_workq",
	};

	mosh_print_version_info();

	k_work_queue_start(
		&mosh_common_work_q,
		mosh_common_workq_stack,
		K_THREAD_STACK_SIZEOF(mosh_common_workq_stack),
		MOSH_COMMON_WORKQ_PRIORITY,
		&cfg);


#if defined(CONFIG_BOOTLOADER_MCUBOOT)
	boot_write_img_confirmed();
#endif
	k_poll_signal_init(&mosh_signal);

	// io_expander = DEVICE_DT_GET(DT_NODELABEL(io_expander));

	// if (!device_is_ready(en_3v2.port)) {
	// 	LOG_ERR("gpio: device not ready.\n");
	// 	return;
	// }

	// if (!device_is_ready(io_expander)) {
	// 	LOG_ERR("tca9534: device not ready.\n");
	// 	return;
	// }

	// ret = gpio_pin_configure_dt(&en_3v2, GPIO_OUTPUT_ACTIVE);
	// if (ret < 0) {
	// 	LOG_ERR("Failed to configure en_3v2 pin");
	// 	return;
	// }

	// ret = gpio_pin_configure(io_expander, USERLED_1, GPIO_OUTPUT);
	// if (ret < 0) {
	// 	LOG_ERR("Failed to configure USERLED_1 pin");
	// 	return;
	// }

	// while (1) {
	// 	ret = gpio_pin_toggle_dt(&en_3v2);
	// 	LOG_DBG("Toggled en_3v2 pin");
	// 	if (ret < 0) {
	// 		LOG_ERR("Failed to toggle en_3v2 pin");
	// 	}

	// 	ret = gpio_pin_toggle(io_expander, USERLED_1);
	// 	LOG_DBG("Toggled USERLED_1 pin");
	// 	if (ret < 0) {
	// 		LOG_ERR("Failed to toggle USERLED_1 pin");
	// 	}

	// 	k_msleep(SLEEP_TIME_MS);
	// }
}